package com.reachauto.vspcloud.logsearch.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.reachauto.vspcloud.common.response.Response;
import com.reachauto.vspcloud.common.response.ResponseHelper;
import com.reachauto.vspcloud.logsearch.bean.vo.FordControlListReq;
import com.reachauto.vspcloud.logsearch.bean.vo.FordControlListSelectVo;
import com.reachauto.vspcloud.logsearch.bean.vo.FordControlListVo;
import com.reachauto.vspcloud.logsearch.bean.vo.FordCrashReq;
import com.reachauto.vspcloud.logsearch.service.FordSearchService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@RestController
@RequestMapping("/api/ford/v1")
@CrossOrigin
public class FordControler {
    @Autowired
    private FordSearchService fordSearchService;

    @RequestMapping(value = "/getVersionList", method = RequestMethod.POST)
    public Response getVersionList() throws IOException {
        Response response = ResponseHelper.createSuccessResponse();
        List<FordControlListSelectVo> versionList = new ArrayList<FordControlListSelectVo>();
        FordControlListSelectVo fordControlListVo1 = new FordControlListSelectVo();
        fordControlListVo1.setName("13.6.4");
        fordControlListVo1.setValue("13.6.4");
        FordControlListSelectVo fordControlListVo2 = new FordControlListSelectVo();
        fordControlListVo2.setName("3.19.12");
        fordControlListVo2.setValue("3.19.12");
        FordControlListSelectVo fordControlListVo3 = new FordControlListSelectVo();
        fordControlListVo3.setName("3.19.8");
        fordControlListVo3.setValue("3.19.8");
        FordControlListSelectVo fordControlListVo4 = new FordControlListSelectVo();
        fordControlListVo4.setName("3.19.4");
        fordControlListVo4.setValue("3.19.4");
        FordControlListSelectVo fordControlListVo5 = new FordControlListSelectVo();
        fordControlListVo5.setName("3.17.1");
        fordControlListVo5.setValue("3.17.1");
        FordControlListSelectVo fordControlListVo6 = new FordControlListSelectVo();
        fordControlListVo6.setName("3.17.2");
        fordControlListVo6.setValue("3.17.2");
        FordControlListSelectVo fordControlListVo7 = new FordControlListSelectVo();
        fordControlListVo7.setName("3.19.1");
        fordControlListVo7.setValue("3.19.1");
        versionList.add(fordControlListVo1);
        versionList.add(fordControlListVo2);
        versionList.add(fordControlListVo3);
        versionList.add(fordControlListVo4);
        versionList.add(fordControlListVo5);
        versionList.add(fordControlListVo6);
        versionList.add(fordControlListVo7);
        response.setResponseBody(JSON.toJSONString(versionList));
        return response;
    }

    /**
     * 查询设备分布
     * 所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getDeviceCrashList", method = RequestMethod.POST)
    public Response getDeviceCrashList(@RequestBody FordControlListReq fordControlListReq) throws IOException {
        Response response = ResponseHelper.createSuccessResponse();
        response = fordSearchService.getDeviceCrashList(fordControlListReq);
        return response;
    }
    /**
     * 系统分布
     * 所选时间范围内，单一系统崩溃次数与总崩溃次数占比top5系统
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getSystemCrashList", method = RequestMethod.POST)
    public Response getSystemCrashList(@RequestBody FordControlListReq fordControlListReq) throws IOException {
        Response response = ResponseHelper.createSuccessResponse();
        response = fordSearchService.getSystemCrashList(fordControlListReq);
        return response;
    }

    /**
     * 版本分布
     * 所选时间范围内，单一版本崩溃次数与总崩溃次数占比top5版本
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getVersionCrashList", method = RequestMethod.POST)
    public Response getVersionCrashList(@RequestBody FordControlListReq fordControlListReq){
        Response response = ResponseHelper.createSuccessResponse();
        response = fordSearchService.getVersionList(fordControlListReq);
        return response;
    }

    /**
     * 根据类型查询崩溃次数折线图
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getCrashList", method = RequestMethod.POST)
    public Response getCrashList(@RequestBody FordCrashReq fordCrashReq){
        Response response = ResponseHelper.createSuccessResponse();
        fordCrashReq.setType("crashSize");
        JSONObject crashSizeResult = null;
        try {
            crashSizeResult = fordSearchService.getCrashList(fordCrashReq);
            fordCrashReq.setType("crashRate");
            JSONObject crashRateResult = fordSearchService.getCrashList(fordCrashReq);
            fordCrashReq.setType("userEffect");
            JSONObject userEffectResult = fordSearchService.getCrashList(fordCrashReq);
            fordCrashReq.setType("userRate");
            JSONObject userRateResult = fordSearchService.getCrashList(fordCrashReq);
            Map<Object,Object> map = new ConcurrentHashMap<Object,Object>();
            map.put("crashSizeResult",crashSizeResult);
            map.put("crashRateResult",crashRateResult);
            map.put("userEffectResult",userEffectResult);
            map.put("userRateResult",userRateResult);
            response.setResponseBody(map);
            return response;
        } catch (ParseException e) {
            response.setCode("000030");
            response.setDescription("");
            e.printStackTrace();
            return response;
        }

    }


// ***************************请求分析模块************************************************************

    /**
     * 1：请求总数按时间范围分组   折线图
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getRequestCountList", method = RequestMethod.POST)
    public Response getRequestCountList(@RequestBody FordControlListReq fordControlListReq){
        Response response = ResponseHelper.createSuccessResponse();
        JSONObject crashSizeResult = null;
        try {
            response = fordSearchService.getRequestCountList(fordControlListReq);
            return response;
        } catch (Exception e) {
            response.setCode("000030");
            response.setDescription("请求总数按时间范围分组出错");
            e.printStackTrace();
            return response;
        }

    }

    /**
     * 2：请求次数Top10的url及请求数量列表  列表
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getRequestCountTopList", method = RequestMethod.POST)
    public Response getRequestCountTopList(@RequestBody FordControlListReq fordControlListReq){
        Response response = ResponseHelper.createSuccessResponse();
        JSONObject crashSizeResult = null;
        try {
            response = fordSearchService.getRequestCountTopList(fordControlListReq);
            return response;
        } catch (Exception e) {
            response.setCode("000030");
            response.setDescription("请求次数Top10的url及请求数量列表");
            e.printStackTrace();
            return response;
        }

    }

    /**
     * 5：请求类型占比以requestMethod为分组  饼图
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getRequestTypeList", method = RequestMethod.POST)
    public Response getRequestTypeList(@RequestBody FordControlListReq fordControlListReq) throws IOException {
        Response response = ResponseHelper.createSuccessResponse();
        response = fordSearchService.getRequestTypeList(fordControlListReq);
        return response;
    }

    /**
     * 6：设备类型请求分布  饼图 以及 折线图   osModel
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getDeriveTypeList", method = RequestMethod.POST)
    public Response getDeriveTypeList(@RequestBody FordControlListReq fordControlListReq) throws IOException {
        Response response = ResponseHelper.createSuccessResponse();
        response = fordSearchService.getDeriveTypeList(fordControlListReq);
        return response;
    }

    /**
     * 3：活跃用户数 --数据
     * 4：活跃用户数  时间范围
     */
    @RequestMapping(value = "/getUserCountList", method = RequestMethod.POST)
    public Response getUserCountList(@RequestBody FordControlListReq fordControlListReq){
        Response response = ResponseHelper.createSuccessResponse();
        JSONObject crashSizeResult = null;
        try {
            response = fordSearchService.getUserCountList(fordControlListReq);
            return response;
        } catch (Exception e) {
            response.setCode("000030");
            response.setDescription("请求总数按时间范围分组出错");
            e.printStackTrace();
            return response;
        }

    }
}
