package com.reachauto.vspcloud.logsearch.controller;

import com.alibaba.fastjson.JSONObject;
import com.reachauto.vspcloud.common.response.Response;
import com.reachauto.vspcloud.common.response.ResponseHelper;
import com.reachauto.vspcloud.logsearch.bean.vo.CrashListReq;
import com.reachauto.vspcloud.logsearch.bean.vo.CrashLogListReq;
import com.reachauto.vspcloud.logsearch.bean.vo.FordControlListReq;
import com.reachauto.vspcloud.logsearch.service.CrashLogSearchService;
import com.reachauto.vspcloud.logsearch.service.ResponseAnalysisService;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@RestController
@RequestMapping("/response-analysis")
@CrossOrigin
public class ResponseAnalysisControler {

    @Autowired
    private ResponseAnalysisService responseAnalysisService;


  /**
   * 请求结果占比,以RequestResult分组
     * @param
   * @return
   * @throws IOException
     */
    @RequestMapping(value = "/getRequestResult", method = RequestMethod.POST)
    public Response getRequestResult() throws IOException {
        return responseAnalysisService.getRequestResult();
    }
//----
  /**
   * 请求结果占比,以RequestResult及时间范围分组
   * @param
   * @return
   * @throws IOException
   */
  @RequestMapping(value = "/getReqResult", method = RequestMethod.POST)
  public Response getReqResult(@RequestBody CrashLogListReq req) throws IOException {
    JSONObject success = responseAnalysisService.getReqResult(req,"Success");
    JSONObject NetError = responseAnalysisService.getReqResult(req,"NetError");
    JSONObject ServerError = responseAnalysisService.getReqResult(req,"ServerError");
    JSONObject RequestError = responseAnalysisService.getReqResult(req,"RequestError");
    Map<Object,Object> map = new HashMap<Object,Object>();
    map.put("success",success);
    map.put("NetError",NetError);
    map.put("ServerError",ServerError);
    map.put("RequestError",RequestError);
    return ResponseHelper.createSuccessResponse(map);
  }

  /**
   * api问题影响用户数
   * @param
   * @return
   * @throws IOException
   */
  @RequestMapping(value = "/getAffectUser", method = RequestMethod.POST)
  public Response getAffectUser(@RequestBody FordControlListReq fordControlListReq) throws IOException {
    JSONObject Success = responseAnalysisService.getAffectUser(fordControlListReq,"Success");
    JSONObject NetError = responseAnalysisService.getAffectUser(fordControlListReq,"NetError");
    JSONObject ServerError = responseAnalysisService.getAffectUser(fordControlListReq,"ServerError");
    JSONObject RequestError = responseAnalysisService.getAffectUser(fordControlListReq,"RequestError");
    Map<Object,Object> map = new HashMap<Object,Object>();
    map.put("success",Success);
    map.put("NetError",NetError);
    map.put("ServerError",ServerError);
    map.put("RequestError",RequestError);
    return ResponseHelper.createSuccessResponse(map);
  }
  /**
   * 平均响应时间最慢的Top10url
   * @param
   * @return
   * @throws IOException
   */
  @RequestMapping(value = "/getTopUrl", method = RequestMethod.POST)
  public Response getTopUrl(@RequestBody CrashLogListReq req) throws IOException {
    return responseAnalysisService.getTopUrl(req);
  }

  /**
   * 响应时间分布
   * @param
   * @return
   * @throws IOException
   */
  @RequestMapping(value = "/getResponseType", method = RequestMethod.POST)
  public Response getResponseType(@RequestBody FordControlListReq fordControlListReq) throws IOException {
    return responseAnalysisService.getResponseType(fordControlListReq);
  }

  /**
   * 响应码占比
   * @param
   * @return
   * @throws IOException
   */
  @RequestMapping(value = "/getHttpCode", method = RequestMethod.POST)
  public Response getHttpCode(@RequestBody FordControlListReq fordControlListReq) throws IOException {
    return responseAnalysisService.getHttpCode(fordControlListReq);
  }

  /**
   * 响应码占比 时间范围
   * @param
   * @return
   * @throws IOException
   */
  @RequestMapping(value = "/getResponseCode", method = RequestMethod.POST)
  public Response getResponseCode(@RequestBody CrashLogListReq req) throws IOException {
    return responseAnalysisService.getResponseCode(req);
  }

}
