package com.reachauto.vspcloud.logsearch.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.reachauto.vspcloud.common.exception.VspBusinessException;
import com.reachauto.vspcloud.common.response.Response;
import com.reachauto.vspcloud.common.response.ResponseHelper;
import com.reachauto.vspcloud.common.utils.StringUtils;
import com.reachauto.vspcloud.constants.ErrorCodeEnum;
import com.reachauto.vspcloud.logsearch.bean.vo.ApiDetailReq;
import com.reachauto.vspcloud.logsearch.service.ApiDetailService;
import java.util.Iterator;
import java.util.Map;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class ApiDetailServiceImpl implements ApiDetailService {

  @Autowired
  private RestHighLevelClient restHighLevelClient;

  @Override
  public Response apiDetailList(ApiDetailReq req) {
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

    BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
    if(StringUtils.isNotBlank(req.getPhoneNum())){
      //PhoneNum
      QueryBuilder queryPhoneNum = QueryBuilders.wildcardQuery("phoneNum", "*"+req.getPhoneNum()+"*");
      boolQueryBuilder.must(queryPhoneNum);
    }
    RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(req.getStartTime()).lte(req.getEndTime()).format("epoch_millis");
    boolQueryBuilder.must(rangeQueryBuilder);
    searchSourceBuilder.query(boolQueryBuilder);
    if (null != req.getPageNo()) {
      searchSourceBuilder.from((req.getPageNo() - 1) * req.getPageSize());
      searchSourceBuilder.size(req.getPageSize());
    } else {
      searchSourceBuilder.size(1000);
    }
    searchSourceBuilder.sort("timestamp", SortOrder.DESC);
    //执行查询
    SearchRequest searchRequest = new SearchRequest("app_api_request_log", "app_api_request_log").source(searchSourceBuilder);
    SearchResponse response = null;

    try {
      response = restHighLevelClient.search(searchRequest);
    } catch (Exception e) {
      e.printStackTrace();
      log.error("api详情日志列表-------------查询es发生异常，入参:{}, Exception:{}", req, e.getStackTrace().toString());
      throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
    }
    SearchHits hits = response.getHits(); // 获取命中次数，查询结果有多少对象
    JSONArray array = new JSONArray();
    JSONObject returnObj = new JSONObject();
    if (hits.getTotalHits() > 0) {
      Iterator<SearchHit> iterator = hits.iterator();
      while (iterator.hasNext()) {
        SearchHit next = iterator.next();
        Map<String, Object> sourceAsMap = next.getSourceAsMap();
        Map<String, Object> map = transferMap(sourceAsMap);
        array.add(map);
      }
    }
    returnObj.put("pageNo", req.getPageNo());
    returnObj.put("pageSize", req.getPageSize());
    returnObj.put("count", hits.getTotalHits());
    returnObj.put("dataList", array);
    return ResponseHelper.createSuccessResponse(returnObj);
  }

  public Map<String, Object> transferMap(Map<String, Object> map){
    map.put("requestrequestTime",map.remove("request.requestTime"));
    map.put("platformosModel",map.remove("platform.osModel"));
    map.put("platformsource",map.remove("platform.source"));
    map.put("requestrequestResult",map.remove("request.requestResult"));
    map.put("platformsysVersion",map.remove("platform.sysVersion"));
    map.put("requestrequestBody",map.remove("request.requestBody"));
    map.put("requesthttpCode",map.remove("request.httpCode"));
    map.put("requesturl",map.remove("request.url"));
    map.put("requestresponseTime",map.remove("request.responseTime"));
    map.put("requestrequestHeader",map.remove("request.requestHeader"));
    map.put("requestrequestMethod",map.remove("request.requestMethod"));
    map.put("requestresponseHeader",map.remove("request.responseHeader"));
    map.put("requestresponseType",map.remove("request.responseType"));
    map.put("platformnetwork",map.remove("platform.network"));
    map.put("requestresponseBody",map.remove("request.responseBody"));
    map.put("platformappVersion",map.remove("platform.appVersion"));

    return map;
  }
}
