package com.reachauto.vspcloud.logsearch.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.reachauto.vspcloud.common.exception.VspBusinessException;
import com.reachauto.vspcloud.common.response.Response;
import com.reachauto.vspcloud.common.response.ResponseHelper;
import com.reachauto.vspcloud.common.utils.StringUtils;
import com.reachauto.vspcloud.constants.ErrorCodeEnum;
import com.reachauto.vspcloud.logsearch.bean.dto.CrashLogDto;
import com.reachauto.vspcloud.logsearch.bean.dto.CrashLogListDto;
import com.reachauto.vspcloud.logsearch.bean.dto.CrashStackDto;
import com.reachauto.vspcloud.logsearch.bean.dto.ListDto;
import com.reachauto.vspcloud.logsearch.bean.vo.CrashListReq;
import com.reachauto.vspcloud.logsearch.bean.vo.CrashStackReq;
import com.reachauto.vspcloud.logsearch.service.CrashLogSearchService;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.collect.HppcMaps.Object.Integer;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.bucket.terms.TermsAggregationBuilder;
import org.elasticsearch.search.aggregations.metrics.sum.ParsedSum;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class CrashLogSearchServiceImpl implements CrashLogSearchService {

  @Autowired
  private RestHighLevelClient restHighLevelClient;

  @Override
  public Response osModelList() throws IOException {
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
    TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("platform.osModel")
        .field("platform.osModel");

    searchSourceBuilder.aggregation(termsBuilder);
    searchSourceBuilder.query();
    //执行查询
    SearchRequest searchRequest = new SearchRequest("app_crash_log", "app_crash_log")
        .source(searchSourceBuilder);
    SearchResponse response = null;

    try {
      response = restHighLevelClient.search(searchRequest);
    } catch (Exception e) {
      e.printStackTrace();
      log.error("crashlog手机型号列表-------------查询es发生异常，入参:{}, Exception:{}",
          e.getStackTrace().toString());
      throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(),
          ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
    }
    Terms terms = response.getAggregations().get("platform.osModel");
    List<ListDto> osModelList = new ArrayList<>();
    for (int i = 0; i < terms.getBuckets().size(); i++) {
      String osModel = terms.getBuckets().get(i).getKey().toString();//id
      ListDto listDto = new ListDto();
      listDto.setName(osModel);
      listDto.setValue(osModel);
      osModelList.add(listDto);
    }
    return ResponseHelper.createSuccessResponse(osModelList);
  }


  @Override
  public Response crashlList(CrashListReq req) {
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
    // 影响用户数
    AggregationBuilder cardinalityPhoneNo = AggregationBuilders.cardinality("phoneNo")
        .field("phoneNo");
    TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("platform.appVersion")
        .field("platform.appVersion");
    // 错误次数
    AggregationBuilder countCrash = AggregationBuilders.count("crashName").field("crashName");

    // 末次发生时间
    AggregationBuilder maxTime = AggregationBuilders.max("maxTime").field("time");

    // 首次发生时间
    AggregationBuilder minTime = AggregationBuilders.min("minTime").field("time");

    TermsAggregationBuilder termsBuilder1 = AggregationBuilders.terms("crashReason")
        .field("crashReason")
        .subAggregation(termsBuilder)
        .subAggregation(cardinalityPhoneNo)
        .subAggregation(countCrash)
        .subAggregation(maxTime)
        .subAggregation(minTime);
    BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
    if(StringUtils.isNotBlank(req.getAppVersion())){
      QueryBuilder queryAppVersion = QueryBuilders.termQuery("platform.appVersion", req.getAppVersion());
      boolQueryBuilder.must(queryAppVersion);
    }
    if(StringUtils.isNotBlank(req.getOsModel())){
      QueryBuilder queryOsModel = QueryBuilders.matchPhraseQuery("platform.osModel", req.getOsModel());
      boolQueryBuilder.must(queryOsModel);
    }
    if(StringUtils.isNotBlank(req.getSysVersion())){
      QueryBuilder querySysVersion = QueryBuilders.matchPhraseQuery("platform.sysVersion", req.getSysVersion());
      boolQueryBuilder.must(querySysVersion);
    }


    RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp")
        .gte(req.getStartTime()).lte(req.getEndTime()).format("epoch_millis");
    boolQueryBuilder.must(rangeQueryBuilder);
    searchSourceBuilder.aggregation(termsBuilder).aggregation(termsBuilder1);
    searchSourceBuilder.query(boolQueryBuilder);

    if (null != req.getPageNo()) {
      searchSourceBuilder.from((req.getPageNo() - 1) * req.getPageSize());
      searchSourceBuilder.size(req.getPageSize());
    } else {
      searchSourceBuilder.size(1000);
    }
    searchSourceBuilder.size(1000);
    searchSourceBuilder.sort("timestamp", SortOrder.DESC);
    //执行查询
    SearchRequest searchRequest = new SearchRequest("app_crash_log", "app_crash_log")
        .source(searchSourceBuilder);
    SearchResponse response = null;

    try {
      response = restHighLevelClient.search(searchRequest);
    } catch (Exception e) {
      e.printStackTrace();
      log.error("crashlog列表-------------查询es发生异常，入参:{}, Exception:{}",
          e.getStackTrace().toString());
      throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(),
          ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
    }
    Terms terms = response.getAggregations().get("crashReason");
    List<CrashLogDto> crashList = new ArrayList<>();
    CrashLogListDto crashLogListDto = new CrashLogListDto();

    SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
    SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    for (int i = 0; i < terms.getBuckets().size(); i++) {
      Terms appVersion = terms.getBuckets().get(i).getAggregations()
          .get("platform.appVersion");
      for (Terms.Bucket entry2 : appVersion.getBuckets()) {
        // 影响用户数
        ParsedSum.SingleValue phoneNoValues = terms.getBuckets().get(i).getAggregations().get("phoneNo");
        String str = phoneNoValues.getValueAsString();
        String valueAsString = str.substring(0, str.indexOf("."));
        // 错误次数
        ParsedSum.SingleValue crashNameValues = terms.getBuckets().get(i).getAggregations().get("crashName");
        String str1 = crashNameValues.getValueAsString();
        String valueAsString1 = str1.substring(0, str1.indexOf("."));
        // 应用版本
        String s = entry2.getKey().toString();
        // 错误摘要
        String crashReason = terms.getBuckets().get(i).getKey().toString();
        // 末次发生时间
        ParsedSum.SingleValue maxTimeValues = terms.getBuckets().get(i).getAggregations().get("maxTime");
        String valueAsString2 = maxTimeValues.getValueAsString();
        String maxTime1 = null;
        try {
          Date date = sdf1.parse(valueAsString2);
          maxTime1 = sdf2.format(date);
        } catch (Exception e) {
          e.printStackTrace();
        }

        // 首次发生时间
        ParsedSum.SingleValue minTimeValues = terms.getBuckets().get(i).getAggregations().get("minTime");
        String valueAsString3 = minTimeValues.getValueAsString();
        String minTime1 = null;
        try {
          Date date = sdf1.parse(valueAsString3);
          minTime1 = sdf2.format(date);
        } catch (Exception e) {
          e.printStackTrace();
        }

        CrashLogDto dto = new CrashLogDto();
        dto.setAppVersion(s);
        dto.setCrashNum(valueAsString1);
        dto.setCrashReason(crashReason);
        dto.setUserNum(valueAsString);
        dto.setMinTime(minTime1);
        dto.setMaxTime(maxTime1);
        crashList.add(dto);
      }
    }
    crashLogListDto.setList(crashList);
    crashLogListDto.setTotalNum(crashList.size());
    return ResponseHelper.createSuccessResponse(crashLogListDto);
  }

  @Override
  public Response sysVersionList() {
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
    TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("platform.sysVersion")
        .field("platform.sysVersion");

    searchSourceBuilder.aggregation(termsBuilder);
    searchSourceBuilder.query();
    //执行查询
    SearchRequest searchRequest = new SearchRequest("app_crash_log", "app_crash_log")
        .source(searchSourceBuilder);
    SearchResponse response = null;

    try {
      response = restHighLevelClient.search(searchRequest);
    } catch (Exception e) {
      e.printStackTrace();
      log.error("crashlog手机系统版本-------------查询es发生异常，入参:{}, Exception:{}",
          e.getStackTrace().toString());
      throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(),
          ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
    }
    Terms terms = response.getAggregations().get("platform.sysVersion");
    List<ListDto> sysVersionList = new ArrayList<>();
    for (int i = 0; i < terms.getBuckets().size(); i++) {
      String sysVersion = terms.getBuckets().get(i).getKey().toString();
      ListDto listDto = new ListDto();
      listDto.setName(sysVersion);
      listDto.setValue(sysVersion);
      sysVersionList.add(listDto);
    }
    return ResponseHelper.createSuccessResponse(sysVersionList);
  }

  @Override
  public Response getCrashStack(CrashStackReq req) {
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
    QueryBuilder appVersion = QueryBuilders.matchPhraseQuery("platform.appVersion", req.getAppVersion());
    QueryBuilder crashReason = QueryBuilders.matchPhraseQuery("crashReason", req.getCrashReason());

    BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(appVersion).must(crashReason);
    searchSourceBuilder.query(boolQueryBuilder);

    searchSourceBuilder.size(2000);
    //执行查询
    searchSourceBuilder.sort("timestamp", SortOrder.DESC);
    SearchRequest searchRequest = new SearchRequest(
        "app_crash_log", "app_crash_log").source(searchSourceBuilder);
    SearchResponse response = null;

    try {
      response = restHighLevelClient.search(searchRequest);
    } catch (Exception e) {
      e.printStackTrace();
      log.error("错误详情列表-------------查询es发生异常，入参:{}, Exception:{}", e.getStackTrace().toString());
      throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
    }
    SearchHits hits = response.getHits(); // 获取命中次数，查询结果有多少对象
    JSONArray array = new JSONArray();
    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
    if (hits.getTotalHits() > 0) {
      Iterator<SearchHit> iterator = hits.iterator();
      while (iterator.hasNext()) {
        Map<String, Object> sourceAsMap = iterator.next().getSourceAsMap();
        CrashStackDto dto = new CrashStackDto();
        Object time = sourceAsMap.get("time");
        Date date = new Date();
        Long aLong = Long.valueOf(String.valueOf(time));
        date.setTime(aLong);
        String format = simpleDateFormat.format(date);
        Object crashStack = sourceAsMap.get("crashStack");
        dto.setCrashStack((String) crashStack);
        dto.setTime(format);
        array.add(dto);
      }
    }
    return ResponseHelper.createSuccessResponse(array);
  }
  }

