package com.reachauto.vspcloud.logsearch.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.reachauto.vspcloud.common.exception.VspBusinessException;
import com.reachauto.vspcloud.common.response.Response;
import com.reachauto.vspcloud.common.response.ResponseHelper;
import com.reachauto.vspcloud.constants.ErrorCodeEnum;
import com.reachauto.vspcloud.logsearch.bean.vo.*;
import com.reachauto.vspcloud.logsearch.service.FordSearchService;
import com.reachauto.vspcloud.util.JdDateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.bucket.histogram.ParsedDateHistogram;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.bucket.terms.TermsAggregationBuilder;
import org.elasticsearch.search.aggregations.metrics.sum.ParsedSum;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.joda.time.DateTimeZone;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.function.ToDoubleFunction;
import java.util.function.ToIntFunction;
import java.util.function.ToLongFunction;

@Slf4j
@Service
public class FordSearchServiceImpl implements FordSearchService {

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    /**
     * 查询设备分布
     * 所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型
     * @return
     * @throws IOException
     */
    @Override
    public Response getDeviceCrashList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        QueryBuilder queryVersion = QueryBuilders.matchPhraseQuery("platform.sysVersion", fordControlListReq.getVersion());
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        if(StringUtils.isNotBlank(fordControlListReq.getVersion())){
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder).must(queryVersion);
            searchSourceBuilder.query(boolQueryBuilder);
        }else{
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
            searchSourceBuilder.query(boolQueryBuilder);
        }
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("platform.osModel").field("platform.osModel")
                .subAggregation(calSum);
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_crash_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型-------------查询es发生异常，入参:{}, Exception:{}", fordControlListReq, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countOsModel = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordControlListVo> list= new ArrayList<FordControlListVo>();
        Terms terms = response.getAggregations().get("platform.osModel");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordControlListVo fordControlListVo = new FordControlListVo();
            String osModel = terms.getBuckets().get(i).getKey().toString();
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            String value = calSum1.getValueAsString();
            countOsModel = new BigDecimal(countOsModel).add(new BigDecimal(value)).doubleValue();
            fordControlListVo.setName(osModel);
            fordControlListVo.setValue(Double.valueOf(value));
            list.add(fordControlListVo);
        }
        for (FordControlListVo fordControlListVo : list) {
            fordControlListVo.setValue(Double.valueOf(String.valueOf(Double.valueOf(fordControlListVo.getValue())/countOsModel*100.00)));
        }
        JSONObject returnObj = new JSONObject();
        returnObj.put("dataList", JSON.toJSONString(list));
        return ResponseHelper.createSuccessResponse(returnObj);
    }
    /**
     * 系统分布
     * 所选时间范围内，单一系统崩溃次数与总崩溃次数占比top5系统
     * @return
     * @throws IOException
     */
    @Override
    public Response getSystemCrashList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        QueryBuilder queryVersion = QueryBuilders.matchPhraseQuery("platform.sysVersion", fordControlListReq.getVersion());
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        if(StringUtils.isNotBlank(fordControlListReq.getVersion())){
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder).must(queryVersion);
            searchSourceBuilder.query(boolQueryBuilder);
        }else{
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
            searchSourceBuilder.query(boolQueryBuilder);
        }
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("platform.sysVersion").field("platform.sysVersion")
                .subAggregation(calSum);
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_crash_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型-------------查询es发生异常，入参:{}, Exception:{}", fordControlListReq, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countOsModel = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordControlListVo> list= new ArrayList<FordControlListVo>();
        Terms terms = response.getAggregations().get("platform.sysVersion");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordControlListVo fordControlListVo = new FordControlListVo();
            String osModel = terms.getBuckets().get(i).getKey().toString();
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            String value = calSum1.getValueAsString();
            countOsModel = new BigDecimal(countOsModel).add(new BigDecimal(value)).doubleValue();
            fordControlListVo.setName(osModel);
            fordControlListVo.setValue(Double.valueOf(value));
            list.add(fordControlListVo);
        }
        for (FordControlListVo fordControlListVo : list) {
            fordControlListVo.setValue(Double.valueOf(String.valueOf(Double.valueOf(fordControlListVo.getValue())/countOsModel*100.00)));
        }
        JSONObject returnObj = new JSONObject();
        returnObj.put("dataList", JSON.toJSONString(list));
        return ResponseHelper.createSuccessResponse(returnObj);
    }
    /**
     * 版本分布
     * 所选时间范围内，单一版本崩溃次数与总崩溃次数占比top5版本
     * @return
     * @throws IOException
     */
    @Override
    public Response getVersionList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        QueryBuilder queryVersion = QueryBuilders.matchPhraseQuery("platform.sysVersion", fordControlListReq.getVersion());
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        if(StringUtils.isNotBlank(fordControlListReq.getVersion())){
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder).must(queryVersion);
            searchSourceBuilder.query(boolQueryBuilder);
        }else{
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
            searchSourceBuilder.query(boolQueryBuilder);
        }
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("platform.appVersion").field("platform.appVersion")
                .subAggregation(calSum);
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_crash_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型-------------查询es发生异常，入参:{}, Exception:{}", fordControlListReq, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countOsModel = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordControlListVo> list= new ArrayList<FordControlListVo>();
        Terms terms = response.getAggregations().get("platform.appVersion");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordControlListVo fordControlListVo = new FordControlListVo();
            String osModel = terms.getBuckets().get(i).getKey().toString();
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            String value = calSum1.getValueAsString();
            countOsModel = new BigDecimal(countOsModel).add(new BigDecimal(value)).doubleValue();
            fordControlListVo.setName(osModel);
            fordControlListVo.setValue(Double.valueOf(value));
            list.add(fordControlListVo);
        }
        for (FordControlListVo fordControlListVo : list) {
            fordControlListVo.setValue(Double.valueOf(String.valueOf(Double.valueOf(fordControlListVo.getValue())/countOsModel*100.00)));
        }
        JSONObject returnObj = new JSONObject();
        returnObj.put("dataList", JSON.toJSONString(list));
        return ResponseHelper.createSuccessResponse(returnObj);
    }

    /**
     * 根据类型查询崩溃次数折线图
     * 1：该类错误在一定时间范围内产生的次数，也就是SDK上报的日志数量  崩溃次数
     * 2: 崩溃率 = 崩溃次数 / 应用启动次数
     * @param fordCrashReq
     * @return
     */
    @Override
    public JSONObject getCrashList(FordCrashReq fordCrashReq) throws ParseException {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        QueryBuilder queryVersion = QueryBuilders.matchPhraseQuery("platform.sysVersion", fordCrashReq.getVersion());
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordCrashReq.getStartTime()).lte(fordCrashReq.getEndTime()).format("epoch_millis");
        if(StringUtils.isNotBlank(fordCrashReq.getVersion())){
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder).must(queryVersion);
            searchSourceBuilder.query(boolQueryBuilder);
        }else{
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
            searchSourceBuilder.query(boolQueryBuilder);
        }
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        //按照手机号去重
        AggregationBuilder calSum2 = AggregationBuilders.cardinality("phoneNo").field("phoneNo");
//        AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
//                .field("timestamp")
//                .interval(fordCrashReq.getEndTime()-fordCrashReq.getStartTime())
//                .timeZone(DateTimeZone.getDefault()).subAggregation(calSum).subAggregation(calSum2);
//        searchSourceBuilder.aggregation(aggregationBuilder);
        if("crashSize".equals(fordCrashReq.getType()) || "crashRate".equals(fordCrashReq.getType())){
            AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
                    .field("timestamp")
                    .interval(JdDateUtil.calInterval(fordCrashReq.getStartTime(),fordCrashReq.getEndTime()))
                    .timeZone(DateTimeZone.getDefault()).subAggregation(calSum);
            searchSourceBuilder.aggregation(aggregationBuilder);
        }else{
            AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
                    .field("timestamp")
                    .interval(JdDateUtil.calInterval(fordCrashReq.getStartTime(),fordCrashReq.getEndTime()))
                    .timeZone(DateTimeZone.getDefault()).subAggregation(calSum).subAggregation(calSum2);
            searchSourceBuilder.aggregation(aggregationBuilder);
        }
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_crash_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型-------------查询es发生异常，入参:{}, Exception:{}", fordCrashReq, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countFordCrash = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordCrashVo> list= new ArrayList<FordCrashVo>();
        ParsedDateHistogram terms = response.getAggregations().get("date_histogram");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordCrashVo fordCrashVo = new FordCrashVo();
            String date = terms.getBuckets().get(i).getKey().toString().substring(0, 10);
//            SimpleDateFormat format0 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//            Date date123 = format0.parse(dateEs);
//            SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd");
//            String date = format1.format(date123.getTime());
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            String value = calSum1.getValueAsString();
            countFordCrash = new BigDecimal(countFordCrash).add(new BigDecimal(value)).doubleValue();
            fordCrashVo.setHorizonated(date);
            fordCrashVo.setOrdinate(value);
            list.add(fordCrashVo);
        }
        if("crashRate".equals(fordCrashReq.getType()) || "userRate".equals(fordCrashReq.getType())){
            for (FordCrashVo fordCrashVo : list) {
                fordCrashVo.setOrdinate(String.valueOf(Double.valueOf(fordCrashVo.getOrdinate())/countFordCrash*100.0));
            }
        }
        List<String> horizonatedList = new ArrayList<>();
        List<String> ordinatedList = new ArrayList<>();
        for (FordCrashVo fordCrashVo : list) {
            horizonatedList.add(fordCrashVo.getHorizonated());
            ordinatedList.add(fordCrashVo.getOrdinate());
        }
        JSONObject returnObj = new JSONObject();
        if("crashSize".equals(fordCrashReq.getType())){
            returnObj.put("size", countFordCrash);
            returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
            returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
        }else if("crashRate".equals(fordCrashReq.getType())){
            returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
            returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
            if(countFordCrash == 0){
                returnObj.put("crashRate", 0);
            }else{
                returnObj.put("crashRate", String.valueOf(Double.valueOf(ordinatedList.size())/countFordCrash*100.00));
            }
        }else if("userEffect".equals(fordCrashReq.getType())){
            returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
            returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
            returnObj.put("userNumber", countFordCrash);
        }else if("userRate".equals(fordCrashReq.getType())){
            returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
            returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
            if(countFordCrash == 0){
                returnObj.put("userRate", 0);
            }else{
                returnObj.put("userRate", String.valueOf(Double.valueOf(ordinatedList.size())/countFordCrash*100.00));
            }
        }
        return returnObj;
    }




// ***************************请求分析模块************************************************************
    /**
     * 1：请求总数按时间范围分组   折线图
     * @return
     * @throws IOException
     */
    @Override
    public Response getRequestCountList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
                    .field("timestamp")
                    .interval(JdDateUtil.calInterval(fordControlListReq.getStartTime(),fordControlListReq.getEndTime()))
                    .timeZone(DateTimeZone.getDefault());
        searchSourceBuilder.aggregation(aggregationBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("请求总数按时间范围分组-------------查询es发生异常，入参:{}, Exception:{}", fordControlListReq, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countFordCrash = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordCrashVo> list= new ArrayList<FordCrashVo>();
        ParsedDateHistogram terms = response.getAggregations().get("date_histogram");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordCrashVo fordCrashVo = new FordCrashVo();
            String date = JdDateUtil.relaceDate(terms.getBuckets().get(i).getKey().toString()).substring(0,19);
            long count = terms.getBuckets().get(i).getDocCount();
            fordCrashVo.setHorizonated(date);
            fordCrashVo.setOrdinate(String.valueOf(count));
            list.add(fordCrashVo);
        }

        List<String> horizonatedList = new ArrayList<>();
        List<String> ordinatedList = new ArrayList<>();
        for (FordCrashVo fordCrashVo : list) {
            horizonatedList.add(fordCrashVo.getHorizonated());
            ordinatedList.add(fordCrashVo.getOrdinate());
        }
        Response response1 = ResponseHelper.createSuccessResponse();
        JSONObject returnObj = new JSONObject();
        returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
        returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
        response1.setResponseBody(returnObj);
        return response1;
    }

    /**
     * 2：请求次数Top10的url及请求数量列表  列表
     * @return
     * @throws IOException
     */
    @Override
    public Response getRequestCountTopList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);;
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.size(10);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.url").field("request.url")
                .subAggregation(calSum).size(10);
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("请求次数Top10的url及请求数量列表  列表-------------查询es发生异常，Exception:{}",e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countFordCrash = 0.0d;
        Map map = new HashMap<String,String>();
        List<RequestCountTopVo> list= new ArrayList<RequestCountTopVo>();
        Terms terms = response.getAggregations().get("request.url");
        Set<String> set = new HashSet<String>();
        Map<String,Long> mapResult = new HashMap<String,Long>();
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            RequestCountTopVo fordControlListVo = new RequestCountTopVo();
            String url = terms.getBuckets().get(i).getKey().toString();
            //ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            long count = terms.getBuckets().get(i).getDocCount();
            set.add(url);
            mapResult.put(url,count);
            fordControlListVo.setCount(count);
            fordControlListVo.setUrl(url);
            list.add(fordControlListVo);
        }
        List<Map.Entry<String, Long>> list123 = new ArrayList<Map.Entry<String, Long>>(mapResult.entrySet());
        list123.sort(new Comparator<Map.Entry<String, Long>>() {
            @Override
            public int compare(Map.Entry<String, Long> o1, Map.Entry<String, Long> o2) {
                return o2.getValue().compareTo(o1.getValue());
            }
        });
//        SearchHits hits = response.getHits(); // 获取命中次数，查询结果有多少对象
        JSONArray array = new JSONArray();
        JSONObject returnObj = new JSONObject();
//        if (hits.getTotalHits() > 0) {
//            Iterator<SearchHit> iterator = hits.iterator();
//            while (iterator.hasNext()) {
//                SearchHit next = iterator.next();
//                Map<String, Object> mapResult = next.getSourceAsMap();
//                Map<String, Object> map123 = transferMap(mapResult);
//                if(set.contains(map123.get("request.url"))){
//                    array.add(next.getSourceAsMap());
//                    set.remove(map123.get("request.url"));
//                }
//            }
//        }
        returnObj.put("dataList", JSON.toJSONString(list));
        Response response1 = ResponseHelper.createSuccessResponse();
        response1.setResponseBody(returnObj);
        return response1;
    }

    public Map<String, Object> transferMap(Map<String, Object> map){
        map.put("requestrequestTime",map.remove("request.requestTime"));
        map.put("platformosModel",map.remove("platform.osModel"));
        map.put("platformsource",map.remove("platform.source"));
        map.put("requestrequestResult",map.remove("request.requestResult"));
        map.put("platformsysVersion",map.remove("platform.sysVersion"));
        map.put("requestrequestBody",map.remove("request.requestBody"));
        map.put("requesthttpCode",map.remove("request.httpCode"));
        map.put("requesturl",map.remove("request.url"));
        map.put("requestresponseTime",map.remove("request.responseTime"));
        map.put("requestrequestHeader",map.remove("request.requestHeader"));
        map.put("requestrequestMethod",map.remove("request.requestMethod"));
        map.put("requestresponseHeader",map.remove("request.responseHeader"));
        map.put("requestresponseType",map.remove("request.responseType"));
        map.put("platformnetwork",map.remove("platform.network"));
        map.put("requestresponseBody",map.remove("request.responseBody"));
        map.put("platformappVersion",map.remove("platform.appVersion"));

        return map;
    }

    /**
     * 5：请求类型占比以requestMethod为分组  饼图
     * @return
     * @throws IOException
     */
    @Override
    public Response getRequestTypeList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.requestMethod").field("request.requestMethod")
                .subAggregation(calSum);
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("请求类型占比以requestMethod为分组  饼图-------------查询es发生异常，入参:, Exception:{}", e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countOsModel = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordControlListVo> list = new ArrayList<FordControlListVo>();
        List<String> listH = new ArrayList<String>();
        List<Double> listO = new ArrayList<Double>();
        Terms terms = response.getAggregations().get("request.requestMethod");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordControlListVo fordControlListVo = new FordControlListVo();
            String requestMethod = terms.getBuckets().get(i).getKey().toString();
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            String value = calSum1.getValueAsString();
            fordControlListVo.setValue(Double.valueOf(value));
            fordControlListVo.setName(requestMethod);
            listH.add(requestMethod);
            listO.add(Double.valueOf(value));
            list.add(fordControlListVo);
            countOsModel = new BigDecimal(countOsModel).add(new BigDecimal(value)).doubleValue();
        }
        JSONObject returnObj = new JSONObject();
        returnObj.put("ordinatedList", JSON.toJSONString(listO));
        returnObj.put("horizonatedList", JSON.toJSONString(listH));
        return ResponseHelper.createSuccessResponse(returnObj);
    }


    /**
     * 6：设备类型请求分布  饼图 以及 折线图   osModel
     * 待优化
     * @return
     * @throws IOException
     */
    @Override
    public Response getDeriveTypeList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        //RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordCrashReq.getStartTime() * 1000).lte(fordCrashReq.getEndTime() * 1000).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll);
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        //AggregationBuilder calSum2 = AggregationBuilders.cardinality("phoneNo").field("phoneNo");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("platform.osModel").field("platform.osModel")
                    .subAggregation(calSum);
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("设备类型请求分布  饼图 以及 折线图   osModel-------------查询es发生异常，入参:, Exception:{}", e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countFordCrash = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordCrashVo> list= new ArrayList<FordCrashVo>();
        Terms terms = response.getAggregations().get("platform.osModel");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordCrashVo fordCrashVo = new FordCrashVo();
            String osModel = terms.getBuckets().get(i).getKey().toString();
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            String value = calSum1.getValueAsString();
            countFordCrash = new BigDecimal(countFordCrash).add(new BigDecimal(value)).doubleValue();
            fordCrashVo.setHorizonated(osModel);
            fordCrashVo.setOrdinate(value);
            list.add(fordCrashVo);
        }
        List<String> horizonatedList = new ArrayList<>();
        List<String> ordinatedList = new ArrayList<>();
        for (FordCrashVo fordCrashVo : list) {
            horizonatedList.add(fordCrashVo.getHorizonated());
            ordinatedList.add(fordCrashVo.getOrdinate());
        }
        JSONObject returnObj = new JSONObject();
        returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
        returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
        return ResponseHelper.createSuccessResponse(returnObj);
    }
    /**
     * 3：活跃用户数 --数据
     * 4：活跃用户数  时间范围
     */
    @Override
    public Response getUserCountList(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum2 = AggregationBuilders.cardinality("phoneNum").field("phoneNum.keyword");
        AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
                .field("timestamp")
                .interval(JdDateUtil.calInterval(fordControlListReq.getStartTime(),fordControlListReq.getEndTime()))
                .timeZone(DateTimeZone.getDefault()).subAggregation(calSum2);
        searchSourceBuilder.aggregation(aggregationBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("活跃用户数  时间范围-------------查询es发生异常，入参:{}, Exception:{}", fordControlListReq, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countFordCrash = 0.0d;
        Map map = new HashMap<String,String>();
        List<FordCrashVo> list= new ArrayList<FordCrashVo>();
        ParsedDateHistogram terms = response.getAggregations().get("date_histogram");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordCrashVo fordCrashVo = new FordCrashVo();
            String date = JdDateUtil.relaceDate(terms.getBuckets().get(i).getKey().toString()).substring(0,19);
            long count = terms.getBuckets().get(i).getDocCount();
            fordCrashVo.setHorizonated(date);
            fordCrashVo.setOrdinate(String.valueOf(count));
            list.add(fordCrashVo);
        }

        List<String> horizonatedList = new ArrayList<>();
        List<String> ordinatedList = new ArrayList<>();
        for (FordCrashVo fordCrashVo : list) {
            horizonatedList.add(fordCrashVo.getHorizonated());
            ordinatedList.add(fordCrashVo.getOrdinate());
        }
        Response response1 = ResponseHelper.createSuccessResponse();
        JSONObject returnObj = new JSONObject();
        returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
        returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
        response1.setResponseBody(returnObj);
        return response1;
    }
}
