package com.reachauto.vspcloud.logsearch.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.reachauto.vspcloud.common.exception.VspBusinessException;
import com.reachauto.vspcloud.common.response.Response;
import com.reachauto.vspcloud.common.response.ResponseHelper;
import com.reachauto.vspcloud.constants.Constants;
import com.reachauto.vspcloud.constants.ErrorCodeEnum;
import com.reachauto.vspcloud.logsearch.bean.vo.RemoteControlLogListReq;
import com.reachauto.vspcloud.logsearch.service.LogSearchService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.elasticsearch.client.reactive.ReactiveElasticsearchClient;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.UUID;

@Slf4j
@Service
public class LogSearchServiceImpl implements LogSearchService {

    @Autowired
    private RestHighLevelClient restHighLevelClient;


    @Override
    public Response remoteControlList(RemoteControlLogListReq req) {

        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        //vin
        QueryBuilder queryVin = QueryBuilders.matchPhraseQuery("reservevalue4.keyword", req.getVin());
        //reservevalue1 000 代表首次进入
        QueryBuilder queryReservevalue1 = QueryBuilders.matchPhraseQuery("reservevalue1.keyword", "000");
        //reservevalue1 0 代表开始
        QueryBuilder queryReservevalue5 = QueryBuilders.matchPhraseQuery("reservevalue5.keyword", "0");
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("@timestamp").gte(req.getStartTime() * 1000).lte(req.getEndTime() * 1000).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(queryVin).must(queryReservevalue1).must(queryReservevalue5).must(rangeQueryBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        if (null != req.getPageNo()) {
            searchSourceBuilder.from((req.getPageNo() - 1) * req.getPageSize());
            searchSourceBuilder.size(req.getPageSize());
        } else {
            searchSourceBuilder.size(1000);
        }
        searchSourceBuilder.sort("@timestamp", SortOrder.DESC);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("vspcloud-app-server-test*", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("控车埋点日志列表-------------查询es发生异常，入参:{}, Exception:{}", req, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        SearchHits hits = response.getHits(); // 获取命中次数，查询结果有多少对象
        JSONArray array = new JSONArray();
        JSONObject returnObj = new JSONObject();
        if (hits.getTotalHits() > 0) {
            Iterator<SearchHit> iterator = hits.iterator();
            while (iterator.hasNext()) {
                SearchHit next = iterator.next();
                array.add(next.getSourceAsMap());
            }
        }
        returnObj.put("pageNo", req.getPageNo());
        returnObj.put("pageSize", req.getPageSize());
        returnObj.put("count", hits.getTotalHits());
        returnObj.put("dataList", array);
        return ResponseHelper.createSuccessResponse(returnObj);
    }

    @Override
    public Response remoteControlDetail(String ptxId, String requestId, Long queryTime) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        QueryBuilder queryPtxId = QueryBuilders.matchPhraseQuery("PtxId.keyword", ptxId);
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("@timestamp").gte((queryTime - 120) * 1000).lte((queryTime + 120) * 1000).format("epoch_millis");


        BoolQueryBuilder boolRequestIdOrPtxid = QueryBuilders.boolQuery().should(queryPtxId);
        if (StringUtils.isNotEmpty(requestId)) {
            //queryRequestId
            QueryBuilder queryRequestId = QueryBuilders.matchPhraseQuery("reservevalue2.keyword", requestId);
            boolRequestIdOrPtxid.should(queryRequestId);
        }

        // 时间范围  and （requestId  or  ptxId   ）
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder).must(boolRequestIdOrPtxid);
        searchSourceBuilder.query(boolQueryBuilder);

        searchSourceBuilder.size(2000);
        //执行查询
        searchSourceBuilder.sort("@timestamp", SortOrder.DESC);
        SearchRequest searchRequest = new SearchRequest(
                "vspcloud-app-server-test*,vspcloud-user-service-test*,vspcloud-vsptrunk-remote-test*,vspcloud-vsptrunk-tcpgateway-test*,vspcloud-vsptrunk-cmcc-test*", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("控车埋点日志列表-------------查询es发生异常，入参:{}, Exception:{}", requestId, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        SearchHits hits = response.getHits(); // 获取命中次数，查询结果有多少对象
        JSONArray array = new JSONArray();
        if (hits.getTotalHits() > 0) {
            Iterator<SearchHit> iterator = hits.iterator();
            while (iterator.hasNext()) {
                SearchHit next = iterator.next();
                array.add(next.getSourceAsMap());
            }
        }
        return ResponseHelper.createSuccessResponse(array);
    }
}
