package com.reachauto.vspcloud.logsearch.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.reachauto.vspcloud.common.exception.VspBusinessException;
import com.reachauto.vspcloud.common.response.Response;
import com.reachauto.vspcloud.common.response.ResponseHelper;
import com.reachauto.vspcloud.constants.ErrorCodeEnum;
import com.reachauto.vspcloud.logsearch.bean.dto.RequestResultDto;
import com.reachauto.vspcloud.logsearch.bean.vo.*;
import com.reachauto.vspcloud.logsearch.service.ResponseAnalysisService;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import com.reachauto.vspcloud.util.JdDateUtil;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.bucket.histogram.ParsedDateHistogram;
import org.elasticsearch.search.aggregations.bucket.terms.ParsedLongTerms;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.bucket.terms.TermsAggregationBuilder;
import org.elasticsearch.search.aggregations.metrics.sum.ParsedSum;
import org.elasticsearch.search.aggregations.metrics.valuecount.ValueCount;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.joda.time.DateTimeZone;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class ResponseAnalysisServiceImpl implements ResponseAnalysisService {

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    @Override
    public Response getRequestResult() {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        AggregationBuilder count = AggregationBuilders.count("timestamp").field("timestamp");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.requestResult")
                .field("request.requestResult")
                .subAggregation(count);

        searchSourceBuilder.aggregation(termsBuilder);
        searchSourceBuilder.query();
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "app_api_request_log")
                .source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("请求结果占比-------------查询es发生异常，入参:{}, Exception:{}",
                    e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(),
                    ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        Terms terms = response.getAggregations().get("request.requestResult");
        List<RequestResultDto> list = new ArrayList<>();
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            String requestResult = terms.getBuckets().get(i).getKey().toString();
            ValueCount valueCount = terms.getBuckets().get(i).getAggregations().get("timestamp");
            Long value = valueCount.getValue();
            RequestResultDto dto = new RequestResultDto();
            dto.setRequestResult(requestResult);
            dto.setCount(value);
            list.add(dto);
        }
        return ResponseHelper.createSuccessResponse(list);
    }

    @Override
    public JSONObject getReqResult(CrashLogListReq req, String type) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        QueryBuilder queryVersion = QueryBuilders.matchPhraseQuery("request.requestResult", type);
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(req.getStartTime()).lte(req.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder).must(queryVersion);
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.requestResult").field("request.requestResult");
        //searchSourceBuilder.aggregation(termsBuilder);
        //按照手机号去重
        //AggregationBuilder calSum2 = AggregationBuilders.cardinality("phoneNo").field("phoneNo");
//        AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
//                .field("timestamp")
//                .interval(fordCrashReq.getEndTime()-fordCrashReq.getStartTime())
//                .timeZone(DateTimeZone.getDefault()).subAggregation(calSum).subAggregation(calSum2);
//        searchSourceBuilder.aggregation(aggregationBuilder);
        AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
                .field("timestamp")
                .interval(JdDateUtil.calInterval(req.getStartTime(), req.getEndTime()))
                .timeZone(DateTimeZone.getDefault()).subAggregation(calSum).subAggregation(termsBuilder);
        searchSourceBuilder.aggregation(aggregationBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型-------------查询es发生异常，入参:{}, Exception:{}", req, e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countFordCrash = 0.0d;
        Map map = new HashMap<String, String>();
        List<FordCrashVo> list = new ArrayList<FordCrashVo>();
        ParsedDateHistogram terms = response.getAggregations().get("date_histogram");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordCrashVo fordCrashVo = new FordCrashVo();
            String date = JdDateUtil.relaceDate(terms.getBuckets().get(i).getKey().toString()).substring(0, 19);
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            //ParsedStringTerms requestResult = terms.getBuckets().get(i).getAggregations().get("request.requestResult");
            String value = calSum1.getValueAsString();
            //String requestResult1 = requestResult.toString();
            countFordCrash = new BigDecimal(countFordCrash).add(new BigDecimal(value)).doubleValue();
            fordCrashVo.setHorizonated(date);
            fordCrashVo.setOrdinate(value);
            //fordCrashVo.setType(requestResult1);
            list.add(fordCrashVo);
        }
        List<String> horizonatedList = new ArrayList<>();
        List<String> ordinatedList = new ArrayList<>();
        //List<String> type = new ArrayList<>();
        for (FordCrashVo fordCrashVo : list) {
            horizonatedList.add(fordCrashVo.getHorizonated());
            ordinatedList.add(fordCrashVo.getOrdinate());
            //type.add(fordCrashVo.getType());
        }
        JSONObject returnObj = new JSONObject();
        //returnObj.put("type", JSON.toJSONString(type));
        returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
        returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
        return returnObj;
    }

    @Override
    public JSONObject getAffectUser(FordControlListReq fordControlListReq, String type) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        QueryBuilder serverError = QueryBuilders.matchPhraseQuery("request.requestResult", type);
        BoolQueryBuilder boolshould = QueryBuilders.boolQuery().should(serverError).should(serverError);
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        SearchSourceBuilder serverSourceBuilder = new SearchSourceBuilder();
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        AggregationBuilder calSum2 = AggregationBuilders.cardinality("phoneNum").field("phoneNum");
        AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
                .field("timestamp")
                .interval(JdDateUtil.calInterval(fordControlListReq.getStartTime(), fordControlListReq.getEndTime()))
                .timeZone(DateTimeZone.getDefault()).subAggregation(calSum).subAggregation(calSum2);
        serverSourceBuilder.aggregation(aggregationBuilder);
        BoolQueryBuilder serverErrorQuery = QueryBuilders.boolQuery().must(rangeQueryBuilder).must(serverError);
        serverSourceBuilder.query(serverErrorQuery);
        SearchRequest searchRequest1 = new SearchRequest("app_api_request_log", "").source(serverSourceBuilder);
        SearchResponse response = null;
        try {
            response = restHighLevelClient.search(searchRequest1);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型-------------查询es发生异常，入参:{}, Exception:{}", e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        ParsedDateHistogram terms1 = response.getAggregations().get("date_histogram");
        List<FordCrashVo> list = new ArrayList<FordCrashVo>();
        for (int i = 0; i < terms1.getBuckets().size(); i++) {
            FordCrashVo fordCrashVo = new FordCrashVo();
            String date = JdDateUtil.relaceDate(terms1.getBuckets().get(i).getKey().toString()).substring(0, 19);
            ParsedSum.SingleValue calSum1 = terms1.getBuckets().get(i).getAggregations().get("phoneNum");
            long count = terms1.getBuckets().get(i).getDocCount();
            fordCrashVo.setHorizonated(date);
            fordCrashVo.setOrdinate(String.valueOf(count));
            list.add(fordCrashVo);
        }
        List<String> horizonatedList = new ArrayList<>();
        List<String> ordinatedList = new ArrayList<>();
        for (FordCrashVo fordCrashVo : list) {
            horizonatedList.add(fordCrashVo.getHorizonated());
            ordinatedList.add(fordCrashVo.getOrdinate());
        }
        JSONObject returnObj = new JSONObject();
        returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
        returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
        return returnObj;
    }

    @Override
    public Response getTopUrl(CrashLogListReq req) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(req.getStartTime()).lte(req.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);

        searchSourceBuilder.size(10);
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.url").field("request.url");
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        searchSourceBuilder.sort("request.responseTime", SortOrder.DESC);
        SearchRequest searchRequest = new SearchRequest(
                "app_api_request_log", "app_api_request_log").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("平均响应时间最慢Top10列表-------------查询es发生异常，入参:{}, Exception:{}", e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        List<String> listDumpliate = new ArrayList<String>();
        List<FordControlListRequestVo> list = new ArrayList<FordControlListRequestVo>();
        Terms terms = response.getAggregations().get("request.url");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            String url = terms.getBuckets().get(i).getKey().toString();
            FordControlListRequestVo fordControlListVo = new FordControlListRequestVo();
            fordControlListVo.setName("url");
            fordControlListVo.setValue(url);
            if (!listDumpliate.contains(url)) {
                list.add(fordControlListVo);
                listDumpliate.add(fordControlListVo.getValue());
            }
        }
        JSONObject returnObj = new JSONObject();
        returnObj.put("dataList", JSON.toJSONString(list));
        return ResponseHelper.createSuccessResponse(returnObj);
    }

    @Override
    public Response getResponseType(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime()).lte(fordControlListReq.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        AggregationBuilder count = AggregationBuilders.count("timestamp").field("timestamp");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.responseType")
                .field("request.responseType")
                .subAggregation(count);

        searchSourceBuilder.aggregation(termsBuilder);
        searchSourceBuilder.query();
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "app_api_request_log")
                .source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("响应时间分布-------------查询es发生异常，入参:{}, Exception:{}",
                    e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(),
                    ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        Terms terms = response.getAggregations().get("request.responseType");
        List<RequestResultDto> list = new ArrayList<>();
        List<Long> ordinatedList = new ArrayList<>();
        List<String> horizonatedList = new ArrayList<>();
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            String requestResult = terms.getBuckets().get(i).getKey().toString();
            ValueCount valueCount = terms.getBuckets().get(i).getAggregations().get("timestamp");
            Long value = valueCount.getValue();
            RequestResultDto dto = new RequestResultDto();
            dto.setRequestResult(requestResult);
            dto.setCount(value);
            ordinatedList.add(value);
            horizonatedList.add(requestResult);
            list.add(dto);
        }
        JSONObject returnObj = new JSONObject();
//        returnObj.put("ordinatedList", JSON.toJSONString(ordinatedList));
//        returnObj.put("horizonatedList", JSON.toJSONString(horizonatedList));
        returnObj.put("ordinatedList", ordinatedList);
        returnObj.put("horizonatedList", horizonatedList);
        return ResponseHelper.createSuccessResponse(returnObj);
    }

    @Override
    public Response getHttpCode(FordControlListReq fordControlListReq) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //queryAll
        QueryBuilder queryAll = QueryBuilders.matchAllQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp").gte(fordControlListReq.getStartTime() * 1000).lte(fordControlListReq.getEndTime() * 1000).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(queryAll).must(rangeQueryBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.size(1000);
        searchSourceBuilder.sort("timestamp", SortOrder.DESC);
        AggregationBuilder calSum = AggregationBuilders.sum("calSum").field("calSum");
        //AggregationBuilder calSum2 = AggregationBuilders.cardinality("phoneNo").field("phoneNo");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.httpCode").field("request.httpCode")
                .subAggregation(calSum);
        searchSourceBuilder.aggregation(termsBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "").source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("httpCode请求分布  饼图 以及 折线图   httpCode请求分布-------------查询es发生异常，入参:, Exception:{}", e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        double countFordCrash = 0.0d;
        Map map = new HashMap<String, String>();
        List<FordCrashVo> list = new ArrayList<FordCrashVo>();
        Terms terms = response.getAggregations().get("request.httpCode");
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            FordCrashVo fordCrashVo = new FordCrashVo();
            String osModel = terms.getBuckets().get(i).getKey().toString();
            ParsedSum.SingleValue calSum1 = terms.getBuckets().get(i).getAggregations().get("calSum");
            String value = calSum1.getValueAsString();
            countFordCrash = new BigDecimal(countFordCrash).add(new BigDecimal(value)).doubleValue();
            fordCrashVo.setHorizonated(osModel);
            fordCrashVo.setOrdinate(value);
            list.add(fordCrashVo);
        }


        //  httpCode--------------------------------------------------
        SearchSourceBuilder serverSourceBuilder = new SearchSourceBuilder();
        TermsAggregationBuilder termsBuilder1 = AggregationBuilders.terms("request.httpCode").field("request.httpCode");
        AggregationBuilder aggregationBuilder = AggregationBuilders.dateHistogram("date_histogram")
                .field("timestamp")
                .interval(JdDateUtil.calInterval(fordControlListReq.getStartTime() * 1000, fordControlListReq.getEndTime() * 1000))
                .timeZone(DateTimeZone.getDefault()).subAggregation(termsBuilder1);
        serverSourceBuilder.aggregation(aggregationBuilder);
        BoolQueryBuilder httpQuery = QueryBuilders.boolQuery().must(rangeQueryBuilder);
        ;
        serverSourceBuilder.query(httpQuery);
        SearchRequest searchRequest1 = new SearchRequest("app_api_request_log", "").source(serverSourceBuilder);
        try {
            response = restHighLevelClient.search(searchRequest1);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("所选时间范围内，单一机型崩溃次数与总崩溃次数占比top5机型-------------查询es发生异常，入参:{}, Exception:{}", e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(), ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        ParsedDateHistogram terms1 = response.getAggregations().get("date_histogram");
        List<JSONObject> list1 = new ArrayList<>();
        List<String> dateList = new ArrayList();
        for (int i = 0; i < terms1.getBuckets().size(); i++) {
            String date = terms1.getBuckets().get(i).getKey().toString();
            ParsedLongTerms parsedLongTerms = terms1.getBuckets().get(i).getAggregations().get("request.httpCode");
            JSONArray array = new JSONArray();
            for (int j = 0; j < parsedLongTerms.getBuckets().size(); j++) {
                JSONObject jsonObject = new JSONObject();
                String name = parsedLongTerms.getBuckets().get(j).getKey().toString();
                jsonObject.put("name", name);
                jsonObject.put("value", parsedLongTerms.getBuckets().get(j).getDocCount());
                array.add(jsonObject);
            }
            JSONObject data = new JSONObject();
            data.put("horizonated",date);
            data.put("ordinate", array);
            dateList.add( UTCToCST(date));
            list1.add(data);
        }
        List<Integer> defaultValueList = new ArrayList<>();
        for (int i = 0; i < dateList.size(); i++) {
            defaultValueList.add(0);
        }
        List<String> horizonatedList = new ArrayList<>();
        List<String> ordinatedList = new ArrayList<>();
        List<JSONObject> valueDataListTemp = new ArrayList<>();
        for (FordCrashVo fordCrashVo : list) {
            horizonatedList.add(fordCrashVo.getHorizonated());
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("name", fordCrashVo.getHorizonated());
            jsonObject.put("value", defaultValueList);
            valueDataListTemp.add(jsonObject);
            ordinatedList.add(fordCrashVo.getOrdinate());
        }
        JSONObject returnObj = new JSONObject();
        List<Object> valueDataList = new ArrayList<>();
        for (int i = 0; i < list1.size(); i++) {
            JSONObject item = list1.get(i);
            for (JSONObject valueItem : valueDataListTemp) {
                String name = valueItem.get("name").toString();
                List<Integer> valueList = (List<Integer>) valueItem.get("value");
                List<Integer> valueListTemp= new ArrayList<>(valueList.size());
                valueListTemp.addAll(valueList);
                for (Object ordinateItem : JSONArray.parseArray(item.get("ordinate").toString())) {
                    JSONObject ordinateJSONItem = JSONObject.parseObject(ordinateItem.toString());
                    if (null != ordinateJSONItem.get("name") && ordinateJSONItem.get("name").equals(name)) {
                        valueListTemp.set(i, ordinateJSONItem.getInteger("value"));
                        valueItem.put("value",valueListTemp);
                        break;
                    }
                }
            }
        }
        for(JSONObject item :valueDataListTemp){
            valueDataList.add(item.get("value"));
        }
//        returnObj.put("lineName", horizonatedList);
//        returnObj.put("dateList", dateList);
//        returnObj.put("valueList", valueDataList);

        for (int i = 0;i<horizonatedList.size();i++) {
            String lineStr  = horizonatedList.get(i);
            JSONObject itemObj = new JSONObject();
            itemObj.put("horizonatedList", dateList);
            itemObj.put("ordinatedList",valueDataList.get(i));

            returnObj.put(lineStr,itemObj);
        }


        return ResponseHelper.createSuccessResponse(returnObj);
    }

    @Override
    public Response getResponseCode(CrashLogListReq req) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        AggregationBuilder count = AggregationBuilders.count("timestamp").field("timestamp");
        TermsAggregationBuilder termsBuilder = AggregationBuilders.terms("request.httpCode")
                .field("request.httpCode")
                .subAggregation(count);
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("timestamp")
                .gte(req.getStartTime()).lte(req.getEndTime()).format("epoch_millis");
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery().must(rangeQueryBuilder);
        searchSourceBuilder.aggregation(termsBuilder);
        searchSourceBuilder.query(boolQueryBuilder);
        //执行查询
        SearchRequest searchRequest = new SearchRequest("app_api_request_log", "app_api_request_log")
                .source(searchSourceBuilder);
        SearchResponse response = null;

        try {
            response = restHighLevelClient.search(searchRequest);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("响应码占比-------------查询es发生异常，入参:{}, Exception:{}",
                    e.getStackTrace().toString());
            throw new VspBusinessException(ErrorCodeEnum.ERR_ES_SEARCH.getCode(),
                    ErrorCodeEnum.ERR_ES_SEARCH.getMsg());
        }
        Terms terms = response.getAggregations().get("request.httpCode");
        List<RequestResultDto> list = new ArrayList<>();
        for (int i = 0; i < terms.getBuckets().size(); i++) {
            String requestResult = terms.getBuckets().get(i).getKey().toString();
            ValueCount valueCount = terms.getBuckets().get(i).getAggregations().get("timestamp");
            Long value = valueCount.getValue();
            RequestResultDto dto = new RequestResultDto();
            Integer integer = Integer.valueOf(requestResult);
            if (integer >= 200 && integer < 299) {
                dto.setRequestResult("Success");
            }
            if ("exception".equals(requestResult)) {
                dto.setRequestResult("NetError");
            }
            if (integer >= 499) {
                dto.setRequestResult("ServerError");
            }
            if (integer >= 299 && integer < 499) {
                dto.setRequestResult("RequestError");
            }
            dto.setCount(value);
            list.add(dto);
        }
        return ResponseHelper.createSuccessResponse(list);
    }
    private String UTCToCST(String UTCStr) {
        String format = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'";
        String format1 = "yyyy-MM-dd HH:mm:ss";
        Date date = null;
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        try {
            date = sdf.parse(UTCStr);
        } catch (ParseException e) {
            date = new Date();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + 8);
        //calendar.getTime() 返回的是Date类型，也可以使用calendar.getTimeInMillis()获取时间戳

        SimpleDateFormat sdf1 = new SimpleDateFormat(format1);
        return sdf1.format(calendar.getTime());
    }
}
