package com.reachauto.vspcloud.util;

import com.google.gson.*;
import com.google.gson.reflect.TypeToken;

import java.lang.reflect.Type;
import java.text.DateFormat;
import java.util.*;
import java.util.Map.Entry;

/**
 * 
 * 
 * <p>
 * Desc: JSON UTIL
 * </p>
 * 
 * @author sunyaoxuan
 * 
 * 
 * @version 1.0
 */
public class JsonUtils {

	private static Gson gson = new Gson();
	private static GsonBuilder gsonBuilder = new GsonBuilder();

	static {
		// 注册时间类型处理 config -> json(带日期类型)
		gson = gsonBuilder.registerTypeAdapter(java.util.Date.class, new DateSerializerUtils())
				.setDateFormat(DateFormat.LONG).create();
		// 注册时间类型处理 json -> Bean(带日期类型)
		gson = gsonBuilder.registerTypeAdapter(java.util.Date.class, new DateDeserializerUtils())
				.setDateFormat(DateFormat.LONG).create();

	}

	/**
	 * 
	 * @Title: toJson @Description: 转成json对象 @param @param src @param @return
	 *         设定文件 @return String 返回类型 @throws
	 */
	public static String toJson(Object object) {
		String json = gson.toJson(object);
		return json;
	}

	@SuppressWarnings("unchecked")
	public static <T> T fromJson(String jsonStr) {
		return (T) gson.fromJson(jsonStr, Object.class);
	}

	/**
	 * 
	 * @Title: fromJson @Description: json 转对象 @param @param
	 *         jsonStr @param @param clz @param @return 设定文件 @return T
	 *         返回类型 @throws
	 */
	public static <T> T fromJson(String jsonStr, Class<T> clz) {

		Objects.requireNonNull(clz, "target Class is null");

		if (jsonStr == null)
			return null;

		T t = gson.fromJson(jsonStr, clz);
		return t;
	}

	public static <T> List<T> fromJsonList(String jsonStr, Class<T> clz) {

		Objects.requireNonNull(clz, "target Class is null");

		List<T> ps = gson.fromJson(jsonStr, new TypeToken<List<T>>() {
		}.getType());

		return ps;
	}
	
	/**
     * 转成list
     * 解决泛型问题
     * @param json
     * @param cls
     * @param <T>
     * @return
     */
    public static <T> List<T> jsonToList(String json, Class<T> cls) {
        Gson gson = new Gson();
        List<T> list = new ArrayList<T>();
        JsonArray array = new JsonParser().parse(json).getAsJsonArray();
        for(final JsonElement elem : array){
            list.add(gson.fromJson(elem, cls));
        }
        return list;

    }

	public static <T> T fromJson(String jsonStr, Type type) {
		T t = gson.fromJson(jsonStr, type);
		return t;
	}

	public static JsonElement replaceKey(JsonElement source, Map<String, String> rep) {
		if (source == null || source.isJsonNull()) {
			return JsonNull.INSTANCE;
		}

		if (source.isJsonPrimitive()) {
			return source;
		}

		if (source.isJsonArray()) {
			JsonArray jsonArr = source.getAsJsonArray();
			JsonArray jsonArray = new JsonArray();
			jsonArr.forEach(item -> {
				jsonArray.add(replaceKey(item, rep));
			});
			return jsonArray;
		}

		if (source.isJsonObject()) {
			JsonObject jsonObj = source.getAsJsonObject();
			Iterator<Entry<String, JsonElement>> iterator = jsonObj.entrySet().iterator();
			JsonObject newJsonObj = new JsonObject();
			iterator.forEachRemaining(item -> {
				String key = item.getKey();
				JsonElement value = item.getValue();
				if (rep.containsKey(key)) {
					String newKey = rep.get(key);
					key = newKey;
				}
				newJsonObj.add(key, replaceKey(value, rep));
			});

			return newJsonObj;
		}
		return JsonNull.INSTANCE;
	}

}
